@@ -18,13 +18,14 @@
 #
 #  SYNOPSIS
 #
-#     elliptic(response, model="linear", times=NULL, dose=NULL,
-#	ccov=NULL, tvcov=NULL, nest=NULL, torder=0, interaction=NULL,
-#	transform="identity", link="identity", autocorr="exponential",
-#	pell=NULL, preg=NULL, pvar=var(y), varfn=NULL, par=NULL, pre=NULL,
-#	delta=NULL, envir=sys.frame(sys.parent()),print.level=0,
-#	ndigit=10, gradtol=0.00001, steptol=0.00001, iterlim=100, fscale=1,
-#	stepmax=10*sqrt(theta%*%theta),typsiz=abs(c(theta)))
+#     elliptic(response, model="linear", distribution="elliptic", times=NULL,
+#	dose=NULL, ccov=NULL, tvcov=NULL, nest=NULL, torder=0,
+#	interaction=NULL, transform="identity", link="identity",
+#	autocorr="exponential", pell=NULL, preg=NULL, pvar=var(y), varfn=NULL,
+#	par=NULL, pre=NULL, delta=NULL, shfn=F, common=F,
+#	envir=sys.frame(sys.parent()), print.level=0, ndigit=10,
+#	gradtol=0.00001, steptol=0.00001, iterlim=100, fscale=1,
+#	stepmax=10*sqrt(theta%*%theta), typsiz=abs(c(theta)))
 #
 #  DESCRIPTION
 #
@@ -32,27 +33,30 @@
 # various autocorrelation functions, one or two levels of random
 # effects, and nonlinear regression.
 
-elliptic <- function(response, model="linear", times=NULL, dose=NULL,
-	ccov=NULL, tvcov=NULL, nest=NULL, torder=0, interaction=NULL,
-	transform="identity", link="identity", autocorr="exponential",
-	pell=NULL, preg=NULL, pvar=var(y), varfn=NULL, par=NULL, pre=NULL,
-	delta=NULL, envir=sys.frame(sys.parent()), print.level=0,
-	ndigit=10, gradtol=0.00001, steptol=0.00001, iterlim=100, fscale=1,
-	stepmax=10*sqrt(theta%*%theta),typsiz=abs(c(theta))){
+elliptic <- function(response, model="linear", distribution="elliptic",
+	times=NULL, dose=NULL, ccov=NULL, tvcov=NULL, nest=NULL,
+	torder=0, interaction=NULL, transform="identity",
+	link="identity", autocorr="exponential", pell=NULL, preg=NULL,
+	pvar=var(y), varfn=NULL, par=NULL, pre=NULL, delta=NULL,
+	shfn=F, common=F, envir=sys.frame(sys.parent()),
+	print.level=0, ndigit=10, gradtol=0.00001, steptol=0.00001,
+	iterlim=100, fscale=1, stepmax=10*sqrt(theta%*%theta),
+	typsiz=abs(c(theta))){
 plra <- function(theta){
 	if(mdl==2)mu <- mu1(theta)
-	if(cvar==1)varn <- sh1(theta[n1:n2])
+	if(cvar==1)varn <- sh1(theta)
 	z <- .Fortran("plra",
 		theta=as.double(theta),
 		like=double(1),
+		dist=as.integer(dst),
 		as.double(rxl),
 		x=as.double(times),
 		as.double(y),
-		tvcov=as.double(response$tvcov$tvcov),
-		ccov=as.double(response$ccov$ccov),
+		tvcov=as.double(resp$tvcov$tvcov),
+		ccov=as.double(resp$ccov$ccov),
 		dose=as.double(dose),
-		nobs=as.integer(response$response$nobs),
-		nest=as.integer(response$response$nest),
+		nobs=as.integer(resp$response$nobs),
+		nest=as.integer(resp$response$nest),
 		lnest=as.integer(lnest),
 		dev=double(nm),
 		nind=as.integer(nind),
@@ -85,18 +89,19 @@ plra <- function(theta){
 	list(like=z$like,res=z$dev,beta=z$beta,betacov=z$betacov)}
 plral <- function(theta){
 	if(mdl==2)mu <- mu1(theta)
-	if(cvar==1)varn <- sh1(theta[n1:n2])
+	if(cvar==1)varn <- sh1(theta)
 	z <- .Fortran("plra",
 		theta=as.double(theta),
 		like=double(1),
+		dist=as.integer(dst),
 		as.double(rxl),
 		x=as.double(times),
 		as.double(y),
-		tvcov=as.double(response$tvcov$tvcov),
-		ccov=as.double(response$ccov$ccov),
+		tvcov=as.double(resp$tvcov$tvcov),
+		ccov=as.double(resp$ccov$ccov),
 		dose=as.double(dose),
-		nobs=as.integer(response$response$nobs),
-		nest=as.integer(response$response$nest),
+		nobs=as.integer(resp$response$nobs),
+		nest=as.integer(resp$response$nest),
 		lnest=as.integer(lnest),
 		dev=double(nm),
 		nind=as.integer(nind),
@@ -132,13 +137,23 @@ if(!is.function(model)&&!inherits(model,"formula")&&!is.null(model))
 	model <- match.arg(model,c("linear","logistic","pkpd"))
 tmp <- c("exponential","gaussian","cauchy","spherical","IOU")
 ar <- match(autocorr <- match.arg(autocorr,tmp),tmp)
+tmp <- c("elliptic","Student t")
+dst <- match(distribution <- match.arg(distribution,tmp),tmp)
 transform <- match.arg(transform,c("identity","exp","square","sqrt","log"))
 tmp <- c("identity","exp","square","sqrt","log")
 lnk <- match(link <- match.arg(link,tmp),tmp)
-npar <- ifelse(!is.null(par),1,0)
+if(common){
+	if(!is.function(model))stop("with common parameters, model must be a function")
+	if(!is.function(varfn))stop("with common parameters, varfn must be a function")
+	pvar <- NULL}
+npar <- length(par)
 tvc <- ifelse(!is.null(tvcov),1,0)
-if(!inherits(response,"repeated")){
-	if(!inherits(response,"response"))response <- restovec(response,times,nest=nest,delta=delta)
+respenv <- inherits(response,"repeated")
+envname <- if(respenv)paste(deparse(substitute(response)))
+	else NULL
+if(!respenv){
+	resp <- if(inherits(response,"response")) response
+		else restovec(response,times,nest=nest,delta=delta)
 	if(is.null(ccov))nccov <- 0
 	else {
 		if(!inherits(ccov,"tccov")){
@@ -164,57 +179,77 @@ if(!inherits(response,"repeated")){
 				else tvcname <- colnames(tvcov[[1]])}
 			tvcov <- tvctomat(tvcov,names=tvcname)}
 		tvc <- ncol(tvcov$tvcov)}
-	response <- rmna(response=response, tvcov=tvcov, ccov=ccov)
+	resp <- rmna(response=resp, tvcov=tvcov, ccov=ccov)
 	if(!is.null(ccov))rm(ccov)
 	if(!is.null(tvcov))rm(tvcov)}
 else {
-	nccov <- if(is.null(response$ccov$ccov)) 0
-		 else  ncol(response$ccov$ccov)
-	tvc <- if(is.null(response$tvcov$tvcov)) 0
-		 else  ncol(response$tvcov$tvcov)}
+	if(is.character(model)){
+		resp <- response
+		if(is.null(ccov))resp$ccov <- NULL
+		else if(inherits(ccov,"formula"))
+			resp$ccov$ccov <- attr(finterp(ccov,envir=response,expand=F,name=paste(deparse(substitute(response)))),"model")[,-1,drop=F]
+		else stop("ccov must be a W&R formula")
+		if(is.null(tvcov))resp$tvcov <- NULL
+		else if(inherits(tvcov,"formula"))
+			resp$tvcov$tvcov <- attr(finterp(tvcov,envir=response,name=paste(deparse(substitute(response)))),"model")[,-1,drop=F]
+		else stop("tvcov must be a W&R formula")}
+	else resp <- rmna(response$response)
+	nccov <- if(is.null(resp$ccov$ccov)) 0
+		else  ncol(resp$ccov$ccov)
+	tvc <- if(is.null(resp$tvcov$tvcov)) 0
+		else  ncol(resp$tvcov$tvcov)}
 if((inherits(envir,"repeated")&&
-	(length(response$response$nobs)!=length(envir$response$nobs)||
-	any(response$response$nobs!=envir$response$nobs)))||
+	(length(resp$response$nobs)!=length(envir$response$nobs)||
+	any(resp$response$nobs!=envir$response$nobs)))||
 	(inherits(envir,"tvcov")&&
-	(length(response$response$nobs)!=length(envir$tvcov$nobs)||
-	any(response$response$nobs!=envir$tvcov$nobs))))
+	(length(resp$response$nobs)!=length(envir$tvcov$nobs)||
+	any(resp$response$nobs!=envir$tvcov$nobs))))
 	stop("response and envir objects are incompatible")
-full <- !is.null(response$ccov$ccov)
-y <- response$response$y
-times <- response$response$times
-nind <- length(response$response$nobs)
-nld <- max(response$response$nobs)
+full <- !is.null(resp$ccov$ccov)
+y <- resp$response$y
+times <- resp$response$times
+nind <- length(resp$response$nobs)
+nld <- max(resp$response$nobs)
 mu <- NULL
 varn <- NULL
 npell <- !is.null(pell)
-if(npell&&pell<=0)stop("The elliptic power parameter must be positive.")
-if(!is.null(pre))npre <- length(pre)
-else npre <- 0
-mu3 <- sh3 <- name <- NULL
-if(inherits(envir,"repeated")||inherits(envir,"tccov")){
-	type <- if(inherits(envir,"repeated"))"repeated"
+if(!npell&&dst==2)stop("An estimate of the degrees of freedom must be supplied")
+if(npell&&pell<=0){
+	if(dst==1)stop("The elliptic power parameter must be positive.")
+	else stop("The degrees of freedom parameter must be positive.")}
+npre <- length(pre)
+mu3 <- sh3 <- NULL
+if(respenv||inherits(envir,"repeated")||inherits(envir,"tccov")||inherits(envir,"tvcov")){
+	type <- if(respenv||inherits(envir,"repeated"))"repeated"
 		else if(inherits(envir,"tccov"))"tccov"
 		else "tvcov"
-	name <- paste(deparse(substitute(envir)))
-	if(inherits(mu,"formula")){
-		mu3 <- finterp(mu)
-		class(mu) <- c(class(mu),type)}
+	if(is.null(envname))envname <- paste(deparse(substitute(envir)))
+	if(inherits(model,"formula")){
+		mu3 <- if(respenv)finterp(model,envir=response,name=envname)
+			else finterp(model,envir=envir,name=envname)
+		class(model) <- c(class(model),type)}
 	else if(is.function(model)){
-		mu3 <- model
-		attributes(mu3) <- attributes(fnenvir(model))
+		tmp <- parse(text=paste(deparse(model))[-1])
 		class(model) <- type
-		model <- fnenvir(model,envir=envir,name=name)}
-	if(inherits(shape,"formula")){
-		sh3 <- finterp(shape)
-		class(shape) <- c(class(shape),type)}
+		model <- if(respenv)fnenvir(model,envir=response,name=envname)
+			else fnenvir(model,envir=envir,name=envname)
+		mu3 <- model
+		if(respenv)attr(mu3,"model") <- tmp}
+	if(inherits(varfn,"formula")){
+		sh3 <- if(respenv)finterp(varfn,envir=response,name=envname)
+			else finterp(varfn,envir=envir,name=envname)
+		class(varfn) <- c(class(varfn),type)}
 	else if(is.function(varfn)){
-		sh3 <- varfn
-		attributes(sh3) <- attributes(fnenvir(varfn))
+		tmp <- parse(text=paste(deparse(varfn))[-1])
 		class(varfn) <- type
-		varfn <- fnenvir(varfn,envir=envir,name=name)}}
+		varfn <- if(respenv)fnenvir(varfn,envir=response,name=envname)
+			else fnenvir(varfn,envir=envir,name=envname)
+		sh3 <- varfn
+		if(respenv)attr(sh3,"model") <- tmp}}
 npr <- length(preg)
 if(inherits(model,"formula")){
-	mu2 <- finterp(model,envir=envir,name=name)
+	mu2 <- if(respenv)finterp(model,envir=response,name=envname)
+		else finterp(model,envir=envir,name=envname)
 	npt1 <- length(attr(mu2,"parameters"))
 	if(is.matrix(attr(mu2,"model"))){
 		if(all(dim(attr(mu2,"model"))==1)){
@@ -244,13 +279,17 @@ else if(is.function(model))mu1 <- model
 else mu1 <- NULL
 if(!is.null(mu1)&&is.null(attributes(mu1))){
 	attributes(mu1) <- if(is.function(model)){
-		if(!inherits(model,"formulafn"))attributes(fnenvir(model))
+		if(!inherits(model,"formulafn")){
+			if(respenv)attributes(fnenvir(model,envir=response))
+			else attributes(fnenvir(model,envir=envir))}
 		else attributes(model)}
-		else attributes(fnenvir(mu1))}
+		else {
+			if(respenv)attributes(fnenvir(mu1,envir=response))
+			else attributes(fnenvir(mu1,envir=envir))}}
 nlp <- if(is.function(mu1))length(attr(mu1,"parameters"))
 	else if(is.null(mu1))NULL
 	else npt1
-if(!is.null(nlp)&&nlp!=npr)
+if(!is.null(nlp)&&!common&&nlp!=npr)
 	stop(paste("preg should have",nlp,"initial estimates"))
 if(is.function(mu1))mdl <- 2
 else if(model=="linear"){
@@ -259,12 +298,15 @@ else if(model=="linear"){
 else if(model=="logistic")mdl <- 3
 else if(model=="pkpd")mdl <- 4
 npvar <- length(pvar)
+n1 <- if(common) 1 else length(preg)+1
+n2 <- length(c(preg,pvar))
 if(inherits(varfn,"formula")){
-	sh2 <- finterp(varfn,envir=envir,name=name)
+	sh2 <- if(respenv)finterp(varfn,envir=response,start=n1,name=envname)
+		else finterp(varfn,envir=envir,start=n1,name=envname)
 	npt2 <- length(attr(sh2,"parameters"))
 	if(is.matrix(attr(sh2,"model"))){
 		if(all(dim(attr(sh2,"model"))==1)){
-			sh1 <- function(p) p[1]*rep(1,n)
+			sh1 <- function(p) p[n1]*rep(1,n)
 			attributes(sh1) <- attributes(sh2)
 			sh2 <- NULL}}
 	else {
@@ -286,18 +328,27 @@ if(inherits(varfn,"formula")){
 		else {
 			sh1 <- sh2
 			rm(sh2)}}}
-else if(is.function(varfn))sh1 <- varfn
+else if(is.function(varfn))sh1 <- if(shfn) function(p) varfn(p[n1:n2],mu1(p))
+		else function(p) varfn(p[n1:n2])
 else sh1 <- NULL
 if(!is.null(sh1)&&is.null(attributes(sh1)))
 	attributes(sh1) <- if(is.function(varfn)){
-		if(!inherits(varfn,"formulafn"))attributes(fnenvir(varfn))
+		if(!inherits(varfn,"formulafn")){
+			if(respenv)attributes(fnenvir(varfn,envir=response))
+			else attributes(fnenvir(varfn,envir=envir))}
 		else attributes(varfn)}
-		else attributes(fnenvir(sh1))
-nlp <- if(is.function(varfn))length(attr(sh1,"parameters"))
+		else {
+			if(respenv)attributes(fnenvir(sh1,envir=response))
+			else attributes(fnenvir(sh1,envir=envir))}
+nlp <- if(is.function(varfn))length(attr(sh1,"parameters"))-shfn
 	else if(is.null(varfn))NULL
+	else if(!inherits(varfn,"formula"))stop("varfn must be a function or formula")
 	else npt2
-if(!is.null(nlp)&&nlp!=npvar)
+if(!is.null(nlp)&&!common&&nlp!=npvar)
 	stop(paste("pvar should have",nlp,"initial estimates"))
+if(common){
+	nlp <- length(unique(c(attr(mu1,"parameters"),attr(sh1,"parameters"))))
+	if(nlp!=npr)stop(paste("with a common parameter model, preg should contain",nlp,"estimates"))}
 if(mdl==1&&!is.null(interaction)){
 	if(length(interaction)!=nccov)
 		stop(paste(nccov,"interactions with time must be specified"))
@@ -306,13 +357,11 @@ if(mdl==1&&!is.null(interaction)){
 	else interaction <- interaction+1}
 else interaction <- rep(1,nccov)
 if(mdl==4&&tvc==0&&is.null(dose))stop("Doses required for PKPD model")
-n1 <- length(preg)+1
-n2 <- length(c(preg,pvar))
-if(!is.null(response$response$nest))lnest <- max(response$response$nest)
+if(!is.null(resp$response$nest))lnest <- max(resp$response$nest)
 else {
 	lnest <- 0
-	response$response$nest <- rep(1,length(y))}
-if(mdl==2&&length(mu1(preg))!=sum(response$response$nobs))
+	resp$response$nest <- rep(1,length(y))}
+if(mdl==2&&length(mu1(preg))!=sum(resp$response$nobs))
 	stop("The mean function must provide an estimate for each observation")
 if(!is.null(times)){
 	if(mdl==1){
@@ -321,13 +370,13 @@ if(!is.null(times)){
 	else ave <- 0}
 else {
 	if(!is.null(par))stop("No times. AR cannot be fitted")
-	else if(torder>0)stop("No times. Time trends cannot be fitted.")
+	if(torder>1)stop("No times. Time trends cannot be fitted.")
 	ave <- 0}
 if(full){
 	rxl <- NULL
 	for(i in 1:nind){
 		tmp <- 1
-		if(full)for(j in 1:ncol(response$ccov$ccov))tmp <- tmp+response$ccov$ccov[i,j]*2^(j-1)
+		if(full)for(j in 1:ncol(resp$ccov$ccov))tmp <- tmp+resp$ccov$ccov[i,j]*2^(j-1)
 		rxl <- c(rxl,tmp)}
 	nxrl <- max(rxl)
 	p <- preg}
@@ -335,7 +384,7 @@ else {
 	nxrl <- 1
 	rxl <- matrix(1,ncol=1,nrow=nind)
 	p <- NULL}
-nm <- sum(response$response$nobs)
+nm <- sum(resp$response$nobs)
 if(transform=="identity")jacob <- 0
 else if(transform=="exp"){
 	jacob <- -sum(y)
@@ -352,9 +401,9 @@ else {
 	else if(transform=="log"){
 		jacob <- sum(log(y))
 		y <- log(y)}}
-if(!is.null(response$response$delta)){
-	if(length(response$response$delta)==1)jacob <- jacob-nm*log(response$response$delta)
-	else jacob <- jacob -sum(log(response$response$delta))}
+if(!is.null(resp$response$delta)){
+	if(length(resp$response$delta)==1)jacob <- jacob-nm*log(resp$response$delta)
+	else jacob <- jacob -sum(log(resp$response$delta))}
 if(mdl==1){
 	if(lnk==1&&is.null(varfn)){
 		npvl <- torder+sum(interaction)+tvc
@@ -379,7 +428,7 @@ if(length(preg)!=npv)
 if(is.null(varfn))cvar <- 0
 else if(is.function(sh1)){
 	cvar <- 1
-	if(length(sh1(pvar))!=sum(response$response$nobs))
+	if(length(sh1(if(common)preg else pvar))!=sum(resp$response$nobs))
 		stop("The variance function or formula must provide an estimate for each observation")}
 else if(varfn=="identity"){
 	if(any(y<0))warning("identity variance function not recommended with negative responses")
@@ -407,6 +456,7 @@ z0 <- nlm(plral, p=theta, hessian=T, print.level=print.level,
 p <- z0$estimate
 z <- plra(p)
 like <- z$like+nm*log(pi)/2+jacob
+pred <- y-z$res
 if(np-npvl==1){
 	nlcov <- 1/z0$hessian
 	nlse <- sqrt(nlcov)}
@@ -439,28 +489,61 @@ if(mdl==1){
 			dimnames(betacorr) <- list(seq(1,npv),seq(1,npv))}}}
 else betacov <- NULL
 sigsq <- p[(npv+1):(npv+npvar)]
-if(!(mdl==4&npvar==4))sigsq <- exp(sigsq)
+if(!(mdl==4&&npvar==4)&&!common)sigsq <- exp(sigsq)
 if(npre>0)tausq <- exp(p[(npv+npvar+1):(npv+npvar+npre)])
 else tausq <- 0
 if(npar>0){
 	rho <- exp(p[npv+npvar+npre+1])
 	if(ar!=5)rho <- rho/(1+rho)}
 else rho <- 0
+if(is.null(pell)&&!common&&length(sigsq)==1&&((npar>0&&
+	autocorr=="exponential")||(npar==0&&length(pre)==1))){
+	nt <- sum(resp$response$nobs)
+	mse <- sigsq*length(y)/(length(y)-npv-npvl*(mdl==1))
+	coef <- NULL
+	if(npar>0)coef <- c(coef,log(-log(rho)))
+	if(length(pre)==1)coef <-
+	c(coef,sqrt(exp(p[npv+npvar+1])/mse))
+	z1 <- .Fortran("resid2",
+		np=as.integer(npar+length(pre)),
+		par=as.double(coef),
+		ave=as.double(ave),
+		pred=as.double(pred),
+		rpred=double(nt),
+		sdr=double(nt),
+		res=double(nt),
+		y=as.double(y),
+		mse=as.double(mse),
+		ns=as.integer(length(resp$response$nobs)),
+		nt=as.integer(nt),
+		model=as.integer(npar),
+		t=as.double(times),
+		nobs=as.integer(resp$response$nobs),
+		nod=as.integer(length(pre)),
+		p=double(length(p)),
+		DUP=F)
+	rpred <- z1$rpred
+	sdr <- z1$sdr
+	rres <- z1$res}
+else rpred <- sdr <- rres <- NULL
 if(!is.null(mu3))mu1 <- mu3
 if(!is.null(sh3))sh1 <- sh3
 z <- list(
 	call=call,
 	model=model,
+	distribution=distribution,
 	mu1=mu1,
 	varfn=varfn,
+	common=common,
 	sh1=sh1,
+	shfn=shfn,
 	autocorr=autocorr,
-	response=response$response,
+	response=resp$response,
 	transform=transform,
 	torder=torder-1,
 	interaction=interaction-1,
-	ccov=response$ccov,
-	tvcov=response$tvcov,
+	ccov=resp$ccov,
+	tvcov=resp$tvcov,
 	full=full,
 	link=link,
 	maxlike=like,
@@ -485,10 +568,15 @@ z <- list(
 	tausq=tausq,
 	rho=rho,
 	residuals=z$res,
+	pred=pred,
+	rpred=rpred,
+	sdrpred=sdr,
+	rresiduals=rres,
 	grad=z0$gradient,
 	iterations=z0$iterations,
 	code=z0$code)
 class(z) <- "elliptic"
+if(!is.null(rpred))class(z) <- c(class(z),"recursive")
 return(z)}
 
 coefficients.elliptic <- function(z) z$coefficients
@@ -499,7 +587,9 @@ print.elliptic <- function(z, digits = max(3, .Options$digits - 3)) {
 	if(!is.null(z$ccov$ccov))nccov <- ncol(z$ccov$ccov)
 	else nccov <- 0
 	if(z$npell==0)cat("\nMultivariate normal distribution\n")
-	else cat("\nMultivariate elliptically-contoured distribution\n")
+	else {
+		if(z$distribution=="elliptical")cat("\nMultivariate elliptically-contoured distribution\n")
+		else cat("\nMultivariate Student t distribution\n")}
 	cat("\nCall:",deparse(z$call),sep="\n")
 	cat("\n")
 	if(z$code>2)cat("Warning: no convergence - error",z$code,"\n\n")
@@ -520,12 +610,13 @@ print.elliptic <- function(z, digits = max(3, .Options$digits - 3)) {
 	cat("AIC               ",z$aic,"\n")
 	cat("Iterations        ",z$iterations,"\n\n")
 	tvc <- !is.null(z$tvcov)
-	if(!inherits(z$mu1,"formulafn"))
+	if(!inherits(z$mu1,"formulafn")){
 		cat("Location parameters\n")
-	if(inherits(z$ccov$linear,"formula"))
-		cat("Linear part: ",deparse(z$ccov$linear),sep="\n")
+		if(inherits(z$ccov$linear,"formula"))
+			cat("Linear part: ",deparse(z$ccov$linear),sep="\n")}
 	if(inherits(z$mu1,"formulafn")){
-		cat("Parameters for location function\n")
+		if(z$common)cat("Location function\n")
+		else cat("Parameters for location function\n")
 		if(!is.null(attr(z$mu1,"formula")))cat(deparse(attr(z$mu1,"formula")),sep="\n")
 		else if(!is.null(attr(z$mu1,"model"))){
 			t <- deparse(attr(z$mu1,"model"))
@@ -536,8 +627,9 @@ print.elliptic <- function(z, digits = max(3, .Options$digits - 3)) {
 				colnames(attr(z$mu1,"model"))
 			else attr(z$mu1,"parameters")
 		coef.table <- cbind(z$coef[1:z$npv], z$nlse[1:z$npv])
-		dimnames(coef.table) <- list(cname, c("estimate", "se"))
-		print.default(coef.table, digits=digits, print.gap=2)}
+		if(!z$common){
+			dimnames(coef.table) <- list(cname, c("estimate", "se"))
+			print.default(coef.table,digits=digits,print.gap=2)}}
 	else if(z$model=="linear"){
 		if(z$npell==1&&z$link=="identity"&&!is.function(z$varfn))
 			cat("(Approximate s.e.)\n")
@@ -586,18 +678,22 @@ print.elliptic <- function(z, digits = max(3, .Options$digits - 3)) {
 				dimnames(coef.table) <- list(cname[3], c("estimate", "se"))
 				print.default(coef.table, digits=digits, print.gap=2)}}}
 	if(inherits(z$sh1,"formulafn")){
-		cat("\nParameters for variance function\n")
+		if(z$common)cat("\nVariance function\n")
+		else {
+			cat("\nVariance function parameters\n")
+			cname <- NULL}
 		if(!is.null(attr(z$sh1,"formula")))cat(deparse(attr(z$sh1,"formula")),sep="\n")
 		else if(!is.null(attr(z$sh1,"model"))){
 			t <- deparse(attr(z$sh1,"model"))
 			t[1] <- sub("expression\\(","",t[1])
 			t[length(t)] <- sub("\\)$","",t[length(t)])
 			cat(t,sep="\n")}
-		cname <- if(is.matrix(attr(z$sh1,"model")))
+		cname <- c(cname,if(is.matrix(attr(z$sh1,"model")))
 				colnames(attr(z$sh1,"model"))
-			else attr(z$sh1,"parameters")
-		coef.table <- cbind(z$coef[(z$npv+1):(z$npv+z$npvar)], z$nlse[(z$npv+1):(z$npv+z$npvar)])
-		dimnames(coef.table) <- list(cname, c("estimate", "se"))
+			else attr(z$sh1,"parameters")[1:(length(attr(z$sh1,"parameters"))-z$shfn)])
+		if(!z$common)coef.table <- cbind(z$coef[(z$npv+1):(z$npv+z$npvar)], z$nlse[(z$npv+1):(z$npv+z$npvar)])
+		else cat("\nCommon parameters\n")
+		dimnames(coef.table) <- list(unique(cname),c("estimate","se"))
 		print.default(coef.table, digits=digits, print.gap=2)}
 	else if(!is.function(z$model)&&!inherits(z$mu1,"formulafn")&&z$model=="pkpd"&&z$npvar==4){
 		cat("\nVariance parameters\n")
@@ -632,9 +728,13 @@ print.elliptic <- function(z, digits = max(3, .Options$digits - 3)) {
 		dimnames(coef.table) <- list("rho",c("estimate","se",""))
 		print.default(coef.table, digits=digits, print.gap=2)}
 	if(z$npell==1){
-		cat("\nElliptical distribution power parameter\n")
 		coef.table <- cbind(z$coef[z$np-z$npvl],z$nlse[z$np-z$npvl],exp(z$coef[z$np-z$npvl]))
-		dimnames(coef.table) <- list("",c("estimate","se","power"))
+		if(z$distribution=="elliptic"){
+			cat("\nElliptic distribution power parameter\n")
+			dimnames(coef.table) <- list("",c("estimate","se","power"))}
+		else {
+			cat("\nDegrees of freedom parameter\n")
+			dimnames(coef.table) <- list("",c("estimate","se","d.f."))}
 		print.default(coef.table, digits=digits, print.gap=2)}
 	if(z$np-z$npvl>1){
 		cat("\nCorrelation matrix of nonlinear parameters\n")
